@@ -2,6 +2,7 @@
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
 use std::borrow::Cow;
+use std::cell::RefCell;
 use std::collections::{BTreeMap, BTreeSet, BinaryHeap, HashMap, HashSet, LinkedList, VecDeque};
 use std::convert::Infallible;
 use std::marker::{PhantomData, PhantomPinned};
@@ -463,6 +464,24 @@ where
     }
 }
 
+impl<T> GetSize for RefCell<T>
+where
+    T: GetSize,
+{
+    fn get_heap_size_with_tracker<Tr: GetSizeTracker>(&self, tracker: Tr) -> (usize, Tr) {
+        // We assume that a `RefCell` holds its data at the stack.
+        // Use try_borrow to avoid panicking if the RefCell is already mutably borrowed
+        match self.try_borrow() {
+            Ok(borrowed) => T::get_heap_size_with_tracker(&*borrowed, tracker),
+            Err(_) => {
+                // If the RefCell is already mutably borrowed, we cannot safely access it.
+                // Return 0 for heap size to avoid panic, though this is a rare edge case.
+                (0, tracker)
+            }
+        }
+    }
+}
+
 impl<T> GetSize for OnceLock<T>
 where
     T: GetSize,
@@ -794,3 +813,40 @@ where
         (size + allocation_size, tracker)
     }
 }
+
+#[cfg(feature = "ordermap")]
+impl<K, V, S> GetSize for ordermap::OrderMap<K, V, S>
+where
+    K: GetSize,
+    V: GetSize,
+    S: std::hash::BuildHasher,
+{
+    fn get_heap_size_with_tracker<Tr: GetSizeTracker>(&self, tracker: Tr) -> (usize, Tr) {
+        let (size, tracker) = self
+            .iter()
+            .fold((0, tracker), |(size, tracker), (key, value)| {
+                let (key_size, tracker) = K::get_heap_size_with_tracker(key, tracker);
+                let (value_size, tracker) = V::get_heap_size_with_tracker(value, tracker);
+                (size + key_size + value_size, tracker)
+            });
+
+        let allocation_size = self.capacity() * <(K, V)>::get_stack_size();
+        (size + allocation_size, tracker)
+    }
+}
+
+#[cfg(feature = "ordermap")]
+impl<T, S> GetSize for ordermap::OrderSet<T, S>
+where
+    T: GetSize,
+{
+    fn get_heap_size_with_tracker<Tr: GetSizeTracker>(&self, tracker: Tr) -> (usize, Tr) {
+        let (size, tracker) = self.iter().fold((0, tracker), |(size, tracker), element| {
+            let (elem_size, tracker) = T::get_heap_size_with_tracker(element, tracker);
+            (size + elem_size, tracker)
+        });
+
+        let allocation_size = self.capacity() * T::get_stack_size();
+        (size + allocation_size, tracker)
+    }
+}
