@@ -21,7 +21,7 @@
 )))]
 // Not needed for 2018 edition and conflicts with `rust_2018_idioms`
 #![doc(test(no_crate_inject))]
-#![doc(html_root_url = "https://docs.rs/serde_with/3.14.1/")]
+#![doc(html_root_url = "https://docs.rs/serde_with/3.15.1/")]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![no_std]
 
@@ -253,24 +253,22 @@
 //! # }
 //! ```
 //!
-//! [`DisplayFromStr`]: https://docs.rs/serde_with/3.14.1/serde_with/struct.DisplayFromStr.html
-//! [`with_prefix!`]: https://docs.rs/serde_with/3.14.1/serde_with/macro.with_prefix.html
-//! [`with_suffix!`]: https://docs.rs/serde_with/3.14.1/serde_with/macro.with_suffix.html
-//! [feature flags]: https://docs.rs/serde_with/3.14.1/serde_with/guide/feature_flags/index.html
-//! [skip_serializing_none]: https://docs.rs/serde_with/3.14.1/serde_with/attr.skip_serializing_none.html
-//! [StringWithSeparator]: https://docs.rs/serde_with/3.14.1/serde_with/struct.StringWithSeparator.html
-//! [user guide]: https://docs.rs/serde_with/3.14.1/serde_with/guide/index.html
+//! [`DisplayFromStr`]: https://docs.rs/serde_with/3.15.1/serde_with/struct.DisplayFromStr.html
+//! [`with_prefix!`]: https://docs.rs/serde_with/3.15.1/serde_with/macro.with_prefix.html
+//! [`with_suffix!`]: https://docs.rs/serde_with/3.15.1/serde_with/macro.with_suffix.html
+//! [feature flags]: https://docs.rs/serde_with/3.15.1/serde_with/guide/feature_flags/index.html
+//! [skip_serializing_none]: https://docs.rs/serde_with/3.15.1/serde_with/attr.skip_serializing_none.html
+//! [StringWithSeparator]: https://docs.rs/serde_with/3.15.1/serde_with/struct.StringWithSeparator.html
+//! [user guide]: https://docs.rs/serde_with/3.15.1/serde_with/guide/index.html
 //! [with-annotation]: https://serde.rs/field-attrs.html#with
-//! [as-annotation]: https://docs.rs/serde_with/3.14.1/serde_with/guide/serde_as/index.html
+//! [as-annotation]: https://docs.rs/serde_with/3.15.1/serde_with/guide/serde_as/index.html
 
 #[cfg(feature = "alloc")]
 extern crate alloc;
 #[doc(hidden)]
-pub extern crate core;
+extern crate core;
 #[doc(hidden)]
-pub extern crate serde;
-#[doc(hidden)]
-pub extern crate serde_derive;
+extern crate serde_core;
 #[cfg(feature = "std")]
 extern crate std;
 
@@ -295,7 +293,9 @@ mod duplicate_key_impls;
 #[cfg(feature = "alloc")]
 mod enum_map;
 #[cfg(feature = "std")]
-mod flatten_maybe;
+/// NOT PUBLIC API
+#[doc(hidden)]
+pub mod flatten_maybe;
 pub mod formats;
 #[cfg(feature = "hex")]
 #[cfg_attr(docsrs, doc(cfg(feature = "hex")))]
@@ -402,10 +402,10 @@ pub(crate) mod prelude {
         option::Option,
         pin::Pin,
         result::Result,
-        str::FromStr,
+        str::{self, FromStr},
         time::Duration,
     };
-    pub use serde::{
+    pub use serde_core::{
         de::{
             Deserialize, DeserializeOwned, DeserializeSeed, Deserializer, EnumAccess,
             Error as DeError, Expected, IgnoredAny, IntoDeserializer, MapAccess, SeqAccess,
@@ -494,13 +494,13 @@ pub use serde_with_macros::*;
 /// # }
 /// ```
 ///
-/// [serde_as]: https://docs.rs/serde_with/3.14.1/serde_with/attr.serde_as.html
+/// [serde_as]: https://docs.rs/serde_with/3.15.1/serde_with/attr.serde_as.html
 pub struct As<T: ?Sized>(PhantomData<T>);
 
 /// Adapter to convert from `serde_as` to the serde traits.
 ///
 /// This is the counter-type to [`As`][].
-/// It can be used whenever a type implementing [`DeserializeAs`]/[`SerializeAs`] is required but the normal [`Deserialize`](::serde::Deserialize)/[`Serialize`](::serde::Serialize) traits should be used.
+/// It can be used whenever a type implementing [`DeserializeAs`]/[`SerializeAs`] is required but the normal [`Deserialize`](::serde_core::Deserialize)/[`Serialize`](::serde_core::Serialize) traits should be used.
 /// Check [`As`] for an example.
 pub struct Same;
 
@@ -514,7 +514,7 @@ pub struct Same;
 /// support, which can be found in some crates.
 ///
 /// If you control the type you want to de/serialize, you can instead use the two derive macros, [`SerializeDisplay`] and [`DeserializeFromStr`].
-/// They properly implement the traits [`serde::Serialize`] and [`serde::Deserialize`] such that user of the type no longer have to use the `serde_as` system.
+/// They properly implement the traits [`Serialize`](::serde_core::Serialize) and [`Deserialize`](::serde_core::Deserialize) such that user of the type no longer have to use the `serde_as` system.
 ///
 /// # Examples
 ///
@@ -555,7 +555,7 @@ pub struct DisplayFromStr;
 /// Use the first format if [`De/Serializer::is_human_readable`], otherwise use the second
 ///
 /// If the second format is not specified, the normal
-/// [`Deserialize`](::serde::Deserialize)/[`Serialize`](::serde::Serialize) traits are used.
+/// [`Deserialize`](::serde_core::Deserialize)/[`Serialize`](::serde_core::Serialize) traits are used.
 ///
 /// # Examples
 ///
@@ -591,8 +591,8 @@ pub struct DisplayFromStr;
 /// assert_eq!(vec![145, 2], rmp_serde::to_vec(&x).unwrap());
 /// # }
 /// ```
-/// [`De/Serializer::is_human_readable`]: serde::Serializer::is_human_readable
-/// [`is_human_readable`]: serde::Serializer::is_human_readable
+/// [`De/Serializer::is_human_readable`]: serde_core::Serializer::is_human_readable
+/// [`is_human_readable`]: serde_core::Serializer::is_human_readable
 pub struct IfIsHumanReadable<H, F = Same>(PhantomData<H>, PhantomData<F>);
 
 /// De/Serialize a [`Option<String>`] type while transforming the empty string to [`None`]
@@ -969,7 +969,7 @@ pub struct BytesOrString;
 /// ```
 ///
 /// [`chrono::Duration`]: ::chrono_0_4::Duration
-/// [feature flag]: https://docs.rs/serde_with/3.14.1/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.1/serde_with/guide/feature_flags/index.html
 pub struct DurationSeconds<
     FORMAT: formats::Format = u64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1101,7 +1101,7 @@ pub struct DurationSeconds<
 /// ```
 ///
 /// [`chrono::Duration`]: ::chrono_0_4::Duration
-/// [feature flag]: https://docs.rs/serde_with/3.14.1/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.1/serde_with/guide/feature_flags/index.html
 pub struct DurationSecondsWithFrac<
     FORMAT: formats::Format = f64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1303,7 +1303,7 @@ pub struct DurationNanoSecondsWithFrac<
 /// [`SystemTime`]: std::time::SystemTime
 /// [`chrono::DateTime<Local>`]: ::chrono_0_4::DateTime
 /// [`chrono::DateTime<Utc>`]: ::chrono_0_4::DateTime
-/// [feature flag]: https://docs.rs/serde_with/3.14.1/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.1/serde_with/guide/feature_flags/index.html
 pub struct TimestampSeconds<
     FORMAT: formats::Format = i64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1445,7 +1445,7 @@ pub struct TimestampSeconds<
 /// [`chrono::DateTime<Local>`]: ::chrono_0_4::DateTime
 /// [`chrono::DateTime<Utc>`]: ::chrono_0_4::DateTime
 /// [NaiveDateTime]: ::chrono_0_4::NaiveDateTime
-/// [feature flag]: https://docs.rs/serde_with/3.14.1/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.1/serde_with/guide/feature_flags/index.html
 pub struct TimestampSecondsWithFrac<
     FORMAT: formats::Format = f64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -2180,13 +2180,38 @@ pub struct TryFromIntoRef<T>(PhantomData<T>);
 #[cfg(feature = "alloc")]
 pub struct BorrowCow;
 
+/// A trait to inspect skipped deserialization errors
+///
+/// The [`VecSkipError`] and [`MapSkipError`] adapters allow to skip values which fail to deserialize.
+/// This trait allows inspecting these errors, for example for logging purposes.
+///
+/// The trait has a single method [`inspect_error`][InspectError::inspect_error], which will be called for each deserialization error.
+/// The default implementation for `()` does nothing.
+///
+/// See the documentation of [`VecSkipError`] and [`MapSkipError`] for usage examples.
+#[cfg(feature = "alloc")]
+pub trait InspectError {
+    /// Inspect a deserialization error which was skipped.
+    fn inspect_error(error: impl serde_core::de::Error);
+}
+
+#[cfg(feature = "alloc")]
+impl InspectError for () {
+    fn inspect_error(_error: impl serde_core::de::Error) {}
+}
+
 /// Deserialize a sequence into `Vec<T>`, skipping elements which fail to deserialize.
 ///
 /// The serialization behavior is identical to `Vec<T>`. This is an alternative to `Vec<T>`
 /// which is resilient against unexpected data.
 ///
+/// You can be notified of skipped elements by providing a type that implements the [`InspectError`] trait.
+/// The second generic argument `I` defaults to `()`, which does nothing.
+///
 /// # Examples
 ///
+/// ## Basic Usage
+///
 /// ```rust
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
@@ -2214,8 +2239,51 @@ pub struct BorrowCow;
 /// assert_eq!(data, serde_json::from_str(source_json).unwrap());
 /// # }
 /// ```
+///
+/// ## Using [`InspectError`](`crate::InspectError`) to log skipped elements
+///
+/// ```rust
+/// # #[cfg(all(feature = "macros", feature = "alloc"))] {
+/// # use serde::{Serialize, Deserialize};
+/// # use serde_with::{serde_as, InspectError, VecSkipError};
+/// # use std::cell::RefCell;
+///
+/// struct ErrorInspector;
+///
+/// thread_local! {
+///     static ERRORS: RefCell<Vec<String>> = RefCell::new(Vec::new());
+/// }
+///
+/// impl InspectError for ErrorInspector {
+///     fn inspect_error(error: impl serde::de::Error) {
+///         ERRORS.with(|errors| errors.borrow_mut().push(error.to_string()));
+///     }
+/// }
+///
+/// #[serde_as]
+/// #[derive(Debug, PartialEq, Deserialize, Serialize)]
+/// struct S {
+///     tag: String,
+///     #[serde_as(as = "VecSkipError<_, ErrorInspector>")]
+///     values: Vec<u8>,
+/// }
+///
+/// let json = r#"{"tag":"type","values":[0, "str", 1, [10, 11], -2, {}, 300]}"#;
+/// let s: S = serde_json::from_str(json).unwrap();
+/// assert_eq!(s.values, vec![0, 1]);
+///
+/// let errors = ERRORS.with(|errors| errors.borrow().clone());
+/// eprintln!("Errors: {errors:#?}");
+/// assert_eq!(errors.len(), 5);
+/// assert!(errors[0].contains("invalid type: string \"str\", expected u8"));
+/// assert!(errors[1].contains("invalid type: sequence, expected u8"));
+/// assert!(errors[2].contains("invalid value: integer `-2`, expected u8"));
+/// assert!(errors[3].contains("invalid type: map, expected u8"));
+/// assert!(errors[4].contains("invalid value: integer `300`, expected u8"));
+/// # }
+/// ```
 #[cfg(feature = "alloc")]
-pub struct VecSkipError<T>(PhantomData<T>);
+pub struct VecSkipError<T, I = ()>(PhantomData<(T, I)>);
 
 /// Deserialize a map, skipping keys and values which fail to deserialize.
 ///
@@ -2223,6 +2291,9 @@ pub struct VecSkipError<T>(PhantomData<T>);
 /// a map. Sometimes a map has heterogeneous keys or values but we only care about some specific
 /// types, and it is desirable to skip entries on errors.
 ///
+/// You can be notified of skipped elements by providing a type that implements the [`InspectError`] trait.
+/// The third generic argument `I` defaults to `()`, which does nothing.
+///
 /// It is especially useful in conjunction to `#[serde(flatten)]` to capture a map mixed in with
 /// other entries which we don't want to exhaust in the type definition.
 ///
@@ -2235,6 +2306,8 @@ pub struct VecSkipError<T>(PhantomData<T>);
 ///
 /// # Examples
 ///
+/// ## Basic Usage
+///
 /// ```rust
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
@@ -2272,8 +2345,51 @@ pub struct VecSkipError<T>(PhantomData<T>);
 /// assert_eq!(data, serde_json::from_str(source_json).unwrap());
 /// # }
 /// ```
+///
+/// ## Using [`InspectError`](`crate::InspectError`) to log skipped elements
+///
+/// ```rust
+/// # #[cfg(all(feature = "macros", feature = "alloc"))] {
+/// # use serde::{Serialize, Deserialize};
+/// # use serde_with::{serde_as, InspectError, MapSkipError};
+/// # use std::collections::BTreeMap;
+/// # use std::cell::RefCell;
+///
+/// struct ErrorInspector;
+///
+/// thread_local! {
+///     static ERRORS: RefCell<Vec<String>> = RefCell::new(Vec::new());
+/// }
+///
+/// impl InspectError for ErrorInspector {
+///     fn inspect_error(error: impl serde::de::Error) {
+///         ERRORS.with(|errors| errors.borrow_mut().push(error.to_string()));
+///     }
+/// }
+///
+/// #[serde_as]
+/// #[derive(Debug, PartialEq, Deserialize, Serialize)]
+/// struct S {
+///     tag: String,
+///     #[serde_as(as = "MapSkipError<_, _, ErrorInspector>")]
+///     values: BTreeMap<String, u8>,
+/// }
+///
+/// let json = r#"{"tag":"type","values":{"valid":42,"invalid": null,"another":"str","nested":[1,2,3]}}"#;
+/// let s: S = serde_json::from_str(json).unwrap();
+/// assert_eq!(s.values.len(), 1);
+/// assert_eq!(s.values.get("valid"), Some(&42));
+///
+/// let errors = ERRORS.with(|errors| errors.borrow().clone());
+/// assert_eq!(errors.len(), 3);
+/// eprintln!("Errors: {errors:#?}");
+/// assert!(errors[0].contains("invalid type: null, expected u8"));
+/// assert!(errors[1].contains("invalid type: string \"str\", expected u8"));
+/// assert!(errors[2].contains("invalid type: sequence, expected u8"));
+/// # }
+/// ```
 #[cfg(feature = "alloc")]
-pub struct MapSkipError<K, V>(PhantomData<(K, V)>);
+pub struct MapSkipError<K, V, I = ()>(PhantomData<(K, V, I)>);
 
 /// Deserialize a boolean from a number
 ///
